part of gtag_interop.html;

/// An interface to a [Google Analytics](https://analytics.google.com) session.
class Analytics extends Service {

  /// Creates a new Google Analytics service.
  Analytics(String trackingId, {this.applicationId = '', this.applicationName = '', this.applicationVersion = ''}):
    super(trackingId);

  /// The application identifier.
  final String applicationId;

  /// The application name.
  final String applicationName;

  /// The application version.
  final String applicationVersion;

  /// Value indicating whether the tracking is enabled.
  bool get enabled => js.getProperty(dom.window, 'ga-disable-$trackingId') != true;
  set enabled(bool value) => js.setProperty(dom.window, 'ga-disable-$trackingId', !value);

  /// Sets a value indicating whether to anonymize IP addresses for all events.
  void anonymizeIP([bool value = true]) => configure(Parameters(anonymize_ip: value)); // ignore: avoid_positional_boolean_parameters

  /// Tracks an exception.
  /// This method is shared between all Gtag services: only one call must be made by the main service to notify all targets.
  void sendException({String description, bool fatal}) => sendEvent(Event.exception, Parameters(
    description: description,
    fatal: fatal
  ));

  /// Tracks a page view.
  void sendPageView(String path, {Uri location, String title}) => configure(Parameters(
    page_location: location?.toString(),
    page_path: path,
    page_title: title
  ));

  /// Tracks a screen view.
  /// This method is shared between all Gtag services: only one call must be made by the main service to notify all targets.
  void sendScreenView(String name) => sendEvent(Event.screenView, Parameters(
    app_id: applicationId,
    app_name: applicationName,
    app_version: applicationVersion,
    screen_name: name
  ));

  /// Tracks a user timing.
  /// This method is shared between all Gtag services: only one call must be made by the main service to notify all targets.
  void sendTiming(String name, int value, {String category, String label}) => sendEvent(Event.timingComplete, Parameters(
    event_category: category,
    event_label: label,
    name: name,
    value: value
  ));

  /// Sets the settings of the tracking cookie.
  void setCookie({String name, String domain, Duration expires}) => configure(Parameters(
    cookie_domain: domain,
    cookie_expires: expires?.inSeconds,
    cookie_name: name
  ));

  /// Sets the identifier representing a user.
  void setUserId(String id) => configure(Parameters(user_id: id));
}
